use std::io;
use rand::Rng;

/*
simple tic-tac-toe game vs. a computer which performs random moves

possible improvements
- better GUI
- computer engine, which never loses
- pvp game mode
- ability to play multiple games in a row
- choose if you want to start or not
*/


fn main() {
    init();
}

fn init(){
    start_dialog(); // say "hello" to player
    let mut start_position = [0,0,0,0,0,0,0,0,0]; // position to start with (0 for an empty spot)
    // 1 for player, 2 for computer
    show_board(&mut start_position);    // shows the current position
    main_loop(&mut start_position); // main game loop
}

fn start_dialog(){  // simple dialogue
    println!("---Welcome to Tic-Tac-Toe!---");
    println!("Please enter your name: ");

    let mut _hello = "Hello ";
    let mut name = String::new();

    io::stdin()
        .read_line(&mut name)
        .expect("Failed to read line");

    println!("{}", _hello.to_owned() + &name);
}

fn show_board(_position_to_show: &mut [i32;9]) {    // shows a 3x3 position
    let mut print_pos = String::from("");

    for n in 0..=8 {
        if n == 3 || n == 6 {   // print strings containing 3 of a row
            println!("{}", print_pos);
            print_pos = String::from("");
        }
        match &_position_to_show[n] {
            &0 => print_pos.push_str("*   "),
            &1 => print_pos.push_str("1   "),
            &2 => print_pos.push_str("o   "),
            _ => println!("This case should not happen."),
        }
        if n == 8 {
            println!("{}", print_pos);
        }
    }
}

fn check_board(_position_to_check: &mut [i32;9]) -> i32{
    // not the best implementation, but a working one for a 3x3 board
    return if _position_to_check[0] != 0 &&
        _position_to_check[0] == _position_to_check[1] &&
        _position_to_check[1] == _position_to_check[2] {
        _position_to_check[0]    //first row
    } else if _position_to_check[0] != 0 &&
        _position_to_check[0] == _position_to_check[3] &&
        _position_to_check[3] == _position_to_check[6] {
        _position_to_check[0]    //left column
    } else if _position_to_check[0] != 0 &&
        _position_to_check[0] == _position_to_check[4] &&
        _position_to_check[4] == _position_to_check[8] {
        _position_to_check[0]    //diagonal top left, bottom right
    } else if _position_to_check[1] != 0 &&
        _position_to_check[1] == _position_to_check[4] &&
        _position_to_check[4] == _position_to_check[7] {
        _position_to_check[1]    //middle column
    } else if _position_to_check[2] != 0 &&
        _position_to_check[2] == _position_to_check[4] &&
        _position_to_check[4] == _position_to_check[6] {
        _position_to_check[2]    //diagonal top right, bottom left
    } else if _position_to_check[3] != 0 &&
        _position_to_check[3] == _position_to_check[4] &&
        _position_to_check[4] == _position_to_check[5] {
        _position_to_check[3]    //second row
    } else if _position_to_check[6] != 0 &&
        _position_to_check[6] == _position_to_check[7] &&
        _position_to_check[7] == _position_to_check[8] {
        _position_to_check[6]    //third row
    } else if _position_to_check[2] != 0 &&
        _position_to_check[2] == _position_to_check[5] &&
        _position_to_check[5] == _position_to_check[8] {
        _position_to_check[2]   //right column
    }
    else if !_position_to_check.contains(&0) {
        3
    }
    else
    {
        0
    }
}

fn main_loop(position: &mut [i32;9]){   // main game loop
    let mut current_position = position;    // current position should be the start position

    while check_board(current_position) == 0 {  // check if game over
        current_position = player_move(current_position);   // player move
        show_board(current_position);   // show the updated board
        println!("-------------------");    // for a cleaner visual look

        if check_board(current_position) != 0{  // check board
            break;
        }

        current_position = computer_move(current_position);
        show_board(current_position);
        println!("-------------------");
    }
    let game_result = check_board(current_position);
    if game_result == 1 {   // player has won
        println!("***Congratulations! You won!***");
    }
    else if game_result == 2 {  // computer has won
        println!("You lost! :(");
    }
    else {  // game_result == 0, draw
        println!("It's a draw!");
    }
}

fn player_move(old_position: &mut [i32;9]) -> &mut [i32;9]{
    let mut input = String::from("");

    while !check_input(&input, old_position) {  // input as long as input is valid
        input.clear();
        println!("Please enter your move (1-9).");
        println!("1 2 3");
        println!("4 5 6");
        println!("7 8 9");
        io::stdin()
            .read_line(&mut input)
            .expect("Failed to read line");
    }

    let new_position = old_position;
    let index = input.trim().parse::<i32>().unwrap();
    let index: usize = index as usize;
    new_position[index-1] = 1;  // update the position
    return new_position;
}

fn check_input(input : &String, old_position: &mut [i32;9]) -> bool{
    // no fancy implementation, but simple and working
    return if input.trim() == "1" || input.trim() == "2" || input.trim() == "3"
        || input.trim() == "4" || input.trim() == "5" || input.trim() == "6"
        || input.trim() == "7" || input.trim() == "8" || input.trim() == "9" {
        // check if input is in range

        let index = input.trim().parse::<i32>().unwrap();
        let index: usize = index as usize;

        if old_position[index-1] == 0 { // return true, if square is not occupied
            true
        } else {
            println!("This square is already occupied!");   // message when square is occupied
            false
        }
    } else {
        if input != "" {    // dont print this message for default value ""
            println!("Input out of range");
        }
        false
    }
}

fn computer_move(old_position: &mut [i32;9]) -> &mut [i32;9]{   // computer should make random moves
    let mut rng = rand::thread_rng();   // initializing rng generator
    let random_number = rng.gen_range(0,8); // random number should be in range (0,8)

    // try and error, better implementation would be to only allow random numbers in range
    while old_position[random_number] != 0 {    // check if random number is valid
        let random_number = rng.gen_range(0,8);
        let random_number: usize = random_number as usize;

        if old_position[random_number] == 0 {
            let new_position = old_position;
            new_position[random_number] = 2;    // update square to be occupied by computer
            return new_position;
        }
    }

    // this case is only relevant if the "first-try" random number is valid
    let new_position = old_position;
    new_position[random_number] = 2;
    return new_position;
}
